% (* :Title: PatRec package *)
% 
% (* :Context: PatRec` *)
% 
% (* :Author: Jean-Luc Bloechle *)
% 
% (* :Summary: 
% 	This package provides various functions useful for the course Pattern Recognition.
% *)
% 
% (* :Package Version: 1.0 *)
% 
% (* :Mathematica Version: 5.2 *)
% 
% (* :Copyright: Copyright 2006, Diva, University Of Fribourg *)
% 
% (* :History:
% 	V. 1.0 April 2006, by Jean-Luc Bloechle.
% *)
% 
% (* :Keywords:
% 	pattern, recognition, covariance
% *)
% 
% (* :Limitations:  *)
% 
% (* :Discussion:  *)

BeginPackage['PatRec']

PatRec::usage = "This package provides various functions useful for the course Pattern Recognition."
MaxPosition::usage="The MaxPosition[list_] function returns the position of the maximum value in the list [1..Length[list]]. If the list is a matrix (list of list), the function returns the index of the list containing the maximum value."
MinPosition::usage="The MaxPosition[list_] function returns the position of the maximum value in the list [1..Length[list]]. If the list is a matrix (list of list), the function returns the index of the list containing the minimum value."
EuclideanDistance::usage="The EuclideanDistance[x_,y_] funtions returns the euclidean distance of two points in a d-dimensional space (a point is represented by a vector = a list)."
Normalize::usage = "The Normalize[vector_] function returns a normalized vector."
Average::usage="The Average[samples_] function computes the mean vector of a list of vectors (list of lists)."
Scatter::usage = "The Scatter[samples_] function computes the scatter matrix of a list of samples. A sample being a list of values, all the samples are represented with a list of lists of values. The computation of the scatter matrix is done the same way as the computation of the covariance matrix, the only difference being that the results of the summations are not divided be the number of samples."
Covariance::usage = "The Covariance[samples_] function computes the covariance matrix of a list of samples. A sample being a list of values, all the samples are represented with a list of lists of values."
Covariance1::usage = "The Covariance[samples_] fuctions computes the covariance matrix as Covariance[samples_] does, but the division factor is Length[samples]-1 instead of Length[samples]."
Draw::usage = "This function lets you draw 1D,2D & 3D graphics, you only need to give a data list or matrix and a point size."
PartitionCS::usage = "The PartitionCS[classSamples_,nbOfPartitions_] function is used to partition class samples data into multiple partition of same size."
JoinCS::usage = "The JoinCS[partitions_] function is used to join previously partitioned class samples data (creation of test and train set from a single set)."
Size::usage = "The Size[s_] function returns the number of samples in the class samples data set. A simple Length function would only return the number of classes (the length of the first list)."

Unprotect[PartitionCS, JoinCS, Average,Size, MaxPosition, MinPosition, EuclideanDistance,MahalanobisDistance,Normalize, Covariance,Covariance1,Scatter,Draw]
Begin['Private']

MaxPosition[x_]:= 
	If[Depth[x]==2,
		Module[{max,pos,i},
      		max=x[[1]];
      		pos=1;
      		For[i=2,i<=Length[x],i++,
      			If[x[[i]]>max,max=x[[i]];pos=i,Null]
      		];
      	pos],
		Module[{max,pos,i,j},
      		max=x[[1,1]];
      		pos=1;
      		For[i=1,i<=Length[x], i++,
        		For[j=1,j<=Length[x[[i]]],j++,
          			If[x[[i,j]]>max,max=x[[i,j]];pos=i,Null]
          		]
          	];
      	pos]
	];     
      
MinPosition[x_]:= 
	If[Depth[x]==2,
		Module[{min,pos,i},
      		min=x[[1]];
      		pos=1;
      		For[i=2,i<=Length[x],i++,
      			If[x[[i]]<min,min=x[[i]];pos=i,Null]
      		];
      	pos],
		Module[{min,pos,i,j},
      		min=x[[1,1]];
      		pos=1;
      		For[i=1,i<=Length[x], i++,
        		For[j=1,j<=Length[x[[i]]],j++,
          			If[x[[i,j]]<min,min=x[[i,j]];pos=i,Null]
          		]
          	];
      	pos]
	];  
      
EuclideanDistance[x_,y_]:=
    Sqrt[Module[
      {difference=x-y},
      difference.difference
      ]];  
      
MahalanobisDistance[x_,y_,inverseCovariance_]:=
    Sqrt[Module[
    	{delta=x-y},
    	delta.inverseCovariance.delta
    	]];          
      
Normalize[x_]:=x/Norm[x];

Covariance[s_]:= Scatter[s,Size[s]];            
Covariance1[s_]:= Scatter[s, Size[s]-1];
Scatter[s_]:= PatRec`Scatter[s,1];                    

Scatter[s_,normalize_]:=
If[Depth[s]==3,
    Module[{mean,size,delta,k}, 
      mean=Mean[s];
      size=Length[s];
      
      Sum[
        delta={s[[k]]-mean};
        Transpose[delta].delta,
        {k,1,size}
       ]/normalize
    ],
    Module[{mean,delta,i,j},
  		mean=Average[s];
  		Sum[
      		Sum[delta={s[[i,j]]-mean};Transpose[delta].delta,
        	{j,1,Length[s[[i]]]}
        	],
      		{i,1,Length[s]}
      	]
    	/normalize                          
    ]
];

Average[s_]:=
	If[Depth[s]==3,
		Mean[s],
		Module[{i,j},Sum[Sum[s[[i,j]],{j,1,Length[s[[i]]]}],{i,1,Length[s]}]/Size[s]]
	];
    
PartitionCS[classSamples_,nbOfPartitions_]:=
  Module[{nbOfClasses, size, samples,i,c,partitions},
    nbOfClasses=Length[classSamples];
    partitions =
      Table[
        samples = classSamples[[c]];
        size=Length[samples];
        Partition[samples,Floor[size/nbOfPartitions]],{c,1,nbOfClasses}
        ];
    Table[partitions[[c,i]],{i,1,nbOfPartitions},{c,1,nbOfClasses}]
    (*partitions[p,c,i]*)
    ];    
    
JoinCS[partitions_]:=
  Module[{nbOfClasses,nbOfPartitions,p,c},
    nbOfPartitions = Length[partitions];
    nbOfClasses=Length[partitions[[1]]];
    Table[
      Apply[Join,Table[partitions[[p,c]],{p,1,nbOfPartitions}]]
      ,{c,1,nbOfClasses}]
    (*partitions[p,c,i]*)
    ];    
    
Size[s_]:=If[Depth[s]==3,Length[s],Sum[Length[s[[i]]],{i,1,Length[s]}]];    

Draw[data_,pointsize_] :=
    Module[{aps=AbsolutePointSize[pointsize],
        d = If[Depth[data]==2,{{data}},If[Depth[data]==3,{data},data]]},
      Show[
        Table[          
          Module[{hue = Hue[Mod[i-1,Length[d]]/Length[d]+0.02,1,0.8],
              length = Length[d[[i]]]}, 
            If[Length[d[[1,1]]]==1,               
              Graphics[
                Table[{hue,aps,Point[{i,d[[i,j,1]]}]},{j,1,length}]],
              If[Length[d[[1,1]]]==2,
                Graphics[Table[{hue,aps,Point[d[[i,j]]]},{j,1,length}]],
                Graphics3D[Table[{hue,aps,Point[d[[i,j]]]},{j,1,length}]]
                ](*If*)
              ](*If*)
            ](*Module*)
          ,{i,1,Length[d]}](*Table*),
        If[
          Length[d[[1,1]]]< 3 ,{Axes->True},{ViewPoint->{1.2,-2.2,0.8},Axes->True}] (*You can modify the 3D ViewPoint here*)
        ](*Show*)
      ](*Module*);

End[]
Protect[PartitionCS, JoinCS,Average, Size, MaxPosition, MinPosition, EuclideanDistance,MahalanobisDistance,Normalize, Covariance,Covariance1,Scatter,Draw]
EndPackage[]

